@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 # vi:ts=4:et
 
+import flaky
 import os.path
 import pycurl
 import unittest
@@ -19,6 +20,7 @@ from . import util
 
 setup_module, teardown_module = appmanager.setup(('app', 8380))
 
+@flaky.flaky(max_runs=3)
 class PostTest(unittest.TestCase):
     def setUp(self):
         self.curl = pycurl.Curl()
@@ -99,15 +101,87 @@ class PostTest(unittest.TestCase):
         }]
         self.check_post(send, expect, 'http://localhost:8380/files')
     
-    def test_post_buffer(self):
-        contents = 'hello, world!'
+    def test_post_byte_buffer(self):
+        contents = util.b('hello, world!')
         send = [
             ('field2', (pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents)),
         ]
         expect = [{
             'name': 'field2',
             'filename': 'uploaded.file',
-            'data': contents,
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_unicode_buffer(self):
+        contents = util.u('hello, world!')
+        send = [
+            ('field2', (pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents)),
+        ]
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_tuple_of_tuples_of_tuples(self):
+        contents = util.u('hello, world!')
+        send = (
+            ('field2', (pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents)),
+        )
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_tuple_of_lists_of_tuples(self):
+        contents = util.u('hello, world!')
+        send = (
+            ['field2', (pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents)],
+        )
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_tuple_of_tuple_of_lists(self):
+        contents = util.u('hello, world!')
+        send = (
+            ('field2', [pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents]),
+        )
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_list_of_tuple_of_tuples(self):
+        contents = util.u('hello, world!')
+        send = [
+            ('field2', (pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents)),
+        ]
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
+        }]
+        self.check_post(send, expect, 'http://localhost:8380/files')
+    
+    def test_post_list_of_list_of_lists(self):
+        contents = util.u('hello, world!')
+        send = [
+            ['field2', [pycurl.FORM_BUFFER, 'uploaded.file', pycurl.FORM_BUFFERPTR, contents]],
+        ]
+        expect = [{
+            'name': 'field2',
+            'filename': 'uploaded.file',
+            'data': 'hello, world!',
         }]
         self.check_post(send, expect, 'http://localhost:8380/files')
     
